@@ -55,6 +55,8 @@ FCITX_CONFIGURATION(
     Option<bool> surroundingText{
         this, "SurroundingText",
         _("Restore typing state from surrounding text"), true};
+    Option<bool> displayUnderline{this, "DisplayUnderline",
+                                  _("Underline the preedit text"), true};
 #ifdef ENABLE_QT
     ExternalOption macroEditor{this, "MacroEditor", _("Macro Editor"),
                                "fcitx://config/addon/unikey/macro"};
