@@ -912,12 +912,16 @@ bool isValidCV(ConSeq c, VowelSeq v) {
 
     VowelSeqInfo &vInfo = VSeqList[v];
 
+    // gi doesn't go with i
+    // qu doesn't go with u, uh
+    // q  doesn't go with any vowel
     if ((c == cs_gi && vInfo.v[0] == vnl_i) ||
-        (c == cs_qu && vInfo.v[0] == vnl_u))
-        return false; // gi doesn't go with i, qu doesn't go with u
+        (c == cs_qu && (vInfo.v[0] == vnl_u || vInfo.v[0] == vnl_uh)) ||
+        (c == cs_q))
+        return false;
 
+    // k can only go with the following vowel sequences
     if (c == cs_k) {
-        // k can only go with the following vowel sequences
         static VowelSeq kVseq[] = {vs_e,   vs_i,    vs_y,  vs_er, vs_eo,
                                    vs_eu,  vs_eru,  vs_ia, vs_ie, vs_ier,
                                    vs_ieu, vs_ieru, vs_nil};
@@ -1267,10 +1271,13 @@ int UkEngine::processHookWithUO(UkKeyEvent &ev) {
     default: // vneHookAll, vneHookUO:
         if (v[0] == vnl_u) {
             if (v[1] == vnl_o || v[1] == vnl_or) {
-                // uo -> uo+ if prefixed by "th"
+                // uo -> uo+ if prefixed by "h", "kh", "th", or stand alone
                 if ((vs == vs_uo || vs == vs_uor) && vEnd == m_current &&
-                    m_buffer[m_current].form == vnw_cv &&
-                    m_buffer[m_current - 2].cseq == cs_th) {
+                    ((m_buffer[m_current].form == vnw_cv &&
+                      (m_buffer[m_current - 2].cseq == cs_h ||
+                       m_buffer[m_current - 2].cseq == cs_kh ||
+                       m_buffer[m_current - 2].cseq == cs_th)) ||
+                     m_buffer[m_current].form == vnw_v)) {
                     newVs = vs_uoh;
                     markChange(vStart + 1);
                     m_buffer[vStart + 1].vnSym = vnl_oh;
@@ -1904,6 +1911,7 @@ int UkEngine::processAppend(UkKeyEvent &ev) {
 //----------------------------------------------------------
 int UkEngine::appendVowel(UkKeyEvent &ev) {
     bool autoCompleted = false;
+    bool complexEvent = false;
 
     m_current++;
     WordInfo &entry = m_buffer[m_current];
@@ -1956,10 +1964,28 @@ int UkEngine::appendVowel(UkKeyEvent &ev) {
     case vnw_v:
     case vnw_cv:
         vs = prev.vseq;
+
         prevTonePos = (m_current - 1) - (VSeqList[vs].len - 1) +
                       getTonePosition(vs, true);
         tone = m_buffer[prevTonePos].tone;
 
+        // u+o/uo+ + u/i -> u+o+ + u/i
+        if ((vs == vs_uoh || vs == vs_uho) &&
+            (lowerSym == vnl_i || lowerSym == vnl_u)) {
+            if (vs == vs_uho) {
+                markChange(m_current - 1);
+                prev.vnSym = vnl_oh;
+                prev.vseq = vs_uhoh;
+            } else {
+                markChange(m_current - 2);
+                m_buffer[m_current - 2].vnSym = vnl_uh;
+                m_buffer[m_current - 2].vseq = vs_uh;
+            }
+
+            vs = vs_uhoh;
+            complexEvent = true;
+        }
+
         if (lowerSym != canSym && tone != 0) // new sym has a tone, but there's
                                              // is already a preceeding tone
             newVs = vs_nil;
@@ -2052,6 +2078,10 @@ int UkEngine::appendVowel(UkKeyEvent &ev) {
         break;
     }
 
+    if (complexEvent) {
+        return 1;
+    }
+
     if (!autoCompleted && (m_pCtrl->charsetId != CONV_CHARSET_UNI_CSTRING) &&
         isalpha(entry.keyCode)) {
         return 0;
